// filename ******** IC3.C ************** 
// Interrupt Input Capture program for the MC68HC812A4   
// Jonathan W. Valvano 3/23/04
// PH0,PH1 are debugging LEDs
// PT3 is input from a keyboard column, with +5V pullup resistor
// PT7 is output to keyboard row

#include <hidef.h>      /* common defines and macros */
#include <mc68hc812a4.h>     /* derivative information */
#pragma LINK_INFO DERIVATIVE "hc12a4"
unsigned char Count;
// -------------IC_Init--------------
// enable input capture 3 interrupts on falling edge
// inputs: none
// outputs: none
// will enable interrutps
void IC_Init(void){ 
asm sei           // make atomic
  DDRT |= 0x80;   // PT7 output
  DDRH |= 0x03;   // PH0,PH1 debugging
  PORTT &= ~0x80; // PT7=0 enable row
  PORTH &= ~0x03; // PH1=PH0=0
  TSCR = 0x80;    // enable TCNT
  TMSK2= 0x33;    // 1us clock
  TIOS &= ~0x08;  // PT3 input capture
  TCTL4 = (TCTL4&0x3F)|0x80; // falling edge
  TMSK1 |= 0x08;  // Arm IC3
  TIOS |= 0x10;   // PT4 output compare
  TMSK1 &= ~0x10; // disarm OC4
  Count = 0;  
asm cli
}

void main(void) {	
  COPCTL = 0;
  IC_Init(); // start IC thread
  for(;;) {	
  }
}
//**interrupt on switch touch PT3
interrupt 11 void IC3Han(void){
  PORTH ^= 0x01;  // profile
  TMSK1 &= ~0x08;   // disarm IC3
  TC4 = TCNT+10000; // 10 ms later
  TMSK1 |= 0x10;    // arm OC4
  TFLG1 = 0x10;     // clear C4F
}
void interrupt 12 OC4Han(void){ 
  PORTH ^= 0x02;  // profile
  TMSK1 &= ~0x10;   // disarm OC4
  TMSK1 |= 0x08;    // arm IC3
  TFLG1 = 0x08;     // clear C3F
  if((PORTT&0x08)==0){
    Count++;
  }
}  
