// filename ******** IC.C ************** 
// Interrupt Input Capture program for the MC68HC812A4   
// Jonathan W. Valvano 3/18/04
// PH0,PH1 are debugging LEDs
// PT3 is input from a keyboard column
// PT7 is output to keyboard row

#include <hidef.h>      /* common defines and macros */
#include <mc68hc812a4.h>     /* derivative information */
#pragma LINK_INFO DERIVATIVE "hc12a4"
unsigned char Count;

// MC68HC812A4
unsigned char LastKey;
// assumes all of PORTT used for this interface 
void Key_Init(void){  
asm sei           // make atomic
  DDRT = 0xC0;    // PT7-PT6 are oc outputs to rows
                  // PT3-PT2 are inputs from columns
  PORTT = 0;      // PT7,PT6 will be zero when output
  TIOS &= ~0x0C;  // PT3,PT2 input capture
  TSCR = 0x80;    // enable TCNT
  TMSK2= 0x33;    // 1us clock
  TCTL4 = 0xA0;   // falling edge, IC2,IC3
  TMSK1 |= 0x0C;  // Arm IC3,IC2
  TFLG1 = 0x0C;   // clear C3F,C2F
  TIOS |= 0x10;   // PT4 output compare
  TMSK1 &= ~0x10; // disarm OC4
  LastKey = 0;    // should be a fifo 
  Count = 0;
asm cli
}
void main(void) {	
  COPCTL = 0;
  DDRH = 0xFF;
  Key_Init(); // start IC thread
  for(;;) {	
  }
}
/* Returns ASCII code for key pressed, 
   return zero if no key pressed
   returns 0xFF if more than one is pressed */
unsigned char numKey;
unsigned char key,column; 
unsigned char KeyScan(void){
  numKey = 0;           // no key pressed yet
  key = 0;              // default value for no key
//*********** Row 3 *******************************
  DDRT = 0x80;  	       // select row 3 
  column = PORTT;       // read columns
  if((column&0x08)==0){  
    key = '1';          // row 3, column 3
    numKey++;
  }
  if((column&0x04)==0){  
    key = '2';          // row 3, column 2
    numKey++;
  }
//*********** Row 2 *******************************
  DDRT = 0x40;  	       // select row 2 
  column = PORTT;       // read columns
  if((column&0x08)==0){  
    key = '4';          // row 2, column 3
    numKey++;
  }
  if((column&0x04)==0){  
    key = '5';          // row 2, column 2
    numKey++;
  }
  if(numKey>1){
    key = 0xFF;         // multiple key error
  }
  return key;
}

void interrupt 10 IC2Han(void){ 
PORTH ^= 0x01;
  TMSK1 &= ~0x0C;   // disarm IC3,IC2
  TC4 = TCNT+10000; // 10 ms later
  TMSK1 |= 0x10;    // arm OC4
  TFLG1 = 0x10;     // clear C4F
}
void interrupt 11 IC3Han(void){ 
PORTH ^= 0x02;
  TMSK1 &= ~0x0C;   // disarm IC3,IC2
  TC4 = TCNT+10000; // 10 ms later
  TMSK1 |= 0x10;    // arm OC4
  TFLG1 = 0x10;     // clear C4F
}
unsigned char mykey;
void interrupt 12 OC4Han(void){
PORTH ^= 0x04;
  mykey = KeyScan();
  if(mykey==0xFF){    // more than one
    TFLG1 = 0x10;     // acknowledge OC4
    TC4 = TC4+10000;  // wait for there to be one
  } else{            
    TMSK1 &= ~0x10;    // disarm OC4
    TMSK1 |= 0x0C;     // arm IC3,IC2
    if(mykey){         // exactly one
      LastKey = mykey; // should be fifo Put
      Count++;
    }
  }
  DDRT = 0xC0;         // all rows zero
  TFLG1 = 0x0C;        // clear C3F,C2F
}
